@@ -6,8 +6,10 @@ import re
 
 __RE_STAGE__ = "__RE_STAGE__"
 __QA_STAGE__ = "__QA_STAGE__"
+__Similarly_Stage__ = "__Similarly_Stage__"
 __SWITCH_STAGE__ = StageType.SWITCH
 __LIB_SWITCH_STAGE__ = "__LIB_SWITCH_STAGE__"
+__Classify_STAGE__ = "__Classify_STAGE__"
 
 #
 __QUESTIONS_LABEL__ = ["__SYS_QUESTION__", "question", "says"]
@@ -261,12 +263,89 @@ class LibSwitchStage(Stage):
         raise RuntimeError
 
 
-def gen_multi_agent(stage_dict: dict):
+class SimilarlyStage(QAStage):
+    pass
+
+
+class ClassifyStage(REStage):
+    __Classify_THRESHOLD__ = "__Classify_THRESHOLD__"
+    __SAVED_NAME__ = "__SAVED_NAME__"
+
+    ##
+    __SAVE_Classify_THRESHOLD__ = "__SAVE_Classify_THRESHOLD__"
+    __SAVE_Classify_PASS__ = "__SAVE_Classify_PASS__"
+    __SAVE_Classify_Best__ = "__SAVE_Classify_Best__"
+    __SAVE_Classify_result__ = "__SAVE_Classify_result__"
+
+    @classmethod
+    def get_classify_text(cls, text, rules):
+        re_dict = {}
+        for (rule, entity_name) in rules:
+            entities = cls.__get_entity__(rule, text)
+            if len(entities) > 0:
+                re_dict[entity_name] = 1.0
+            else:
+                re_dict[entity_name] = 0.0
+        return re_dict
+
+    def __init__(self, data):
+        super().__init__(data)
+        self.__Classify_THRESHOLD__ = data.get(self.__Classify_THRESHOLD__, 0.5)
+        self.__SAVED_NAME__ = data.get("__SAVED_NAME__", {})
+
+    def is_fit_needs_n_gen_entity(self, kwargs) -> (bool, dict):
+
+        user_text = kwargs.get(__USER_TEXT__, "")
+        pass_token = False
+
+        classify_values_dict = self.get_classify_text(text=user_text, rules=self.is_fits)
+        for entity_name, value in classify_values_dict.items():
+            if value >= self.__Classify_THRESHOLD__:
+                pass_token = True
+            kwargs = self.set_default_var(kwargs, entity_name, value)
+
+        # Save
+        __LABEL_SAVE_Classify_THRESHOLD__ = self.__SAVED_NAME__.get(self.__SAVE_Classify_THRESHOLD__,
+                                                                    self.__SAVE_Classify_THRESHOLD__)
+        __LABEL_SAVE_Classify_PASS__ = self.__SAVED_NAME__.get(self.__SAVE_Classify_PASS__, self.__SAVE_Classify_PASS__)
+        __LABEL_SAVE_Classify_Best__ = self.__SAVED_NAME__.get(self.__SAVE_Classify_Best__, self.__SAVE_Classify_Best__)
+        __LABEL_SAVE_Classify_result__ = self.__SAVED_NAME__.get(self.__SAVE_Classify_result__,
+                                                                 self.__SAVE_Classify_result__)
+
+        ##
+        kwargs = self.set_default_var(kwargs, __LABEL_SAVE_Classify_result__, classify_values_dict)
+        kwargs = self.set_default_var(kwargs, __LABEL_SAVE_Classify_THRESHOLD__, self.__Classify_THRESHOLD__)
+        kwargs = self.set_default_var(kwargs, __LABEL_SAVE_Classify_PASS__, pass_token)
+        kwargs = self.set_default_var(kwargs, __LABEL_SAVE_Classify_Best__,
+                                      max(classify_values_dict, key=classify_values_dict.get))
+
+        if self.disable_refuse_question:
+            pass_token = True
+
+        return pass_token, kwargs
+
+
+##
+
+#
+__LIB_STAGES__ = {
+    __RE_STAGE__: REStage,
+    __QA_STAGE__: SimilarlyStage,
+    __SWITCH_STAGE__: LibSwitchStage,
+    __LIB_SWITCH_STAGE__: LibSwitchStage,
+    __Similarly_Stage__: SimilarlyStage,
+    __Classify_STAGE__: ClassifyStage
+}
+
+
+def gen_multi_agent(stage_dict: dict, stages_classes=None):
+    if stages_classes is None:
+        stages_classes = __LIB_STAGES__
     stages = {}
     for stages_label, stage_jsons in stage_dict.items():
         stages[stages_label] = []
         for stage_json in stage_jsons:
-            stage_class = __LIB_STAGES__[stage_json["stage_type"]]
+            stage_class = stages_classes[stage_json["stage_type"]]
             stages[stages_label].append(stage_class(stage_json))
 
     return MultiAgent(stages)
@@ -274,12 +353,3 @@ def gen_multi_agent(stage_dict: dict):
 
 def gen_agent(stage_dict: dict):
     return gen_multi_agent(stage_dict)
-
-
-#
-__LIB_STAGES__ = {
-    __RE_STAGE__: REStage,
-    __QA_STAGE__: QAStage,
-    __SWITCH_STAGE__: LibSwitchStage,
-    __LIB_SWITCH_STAGE__: LibSwitchStage
-}
